// 
// The LibSea Graph Library.
// Copyright (C) 2000,2001,2002 The Regents of the University of California.
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// ######END_HEADER######
// 


package org.caida.libsea;

public class ObjectArray extends HeapArray
{
    ////////////////////////////////////////////////////////////////////////
    // CONSTRUCTORS
    ////////////////////////////////////////////////////////////////////////

    public ObjectArray()
    {
	super(1);
    }

    // {segmentSize} must be a power of 2.
    public ObjectArray(int segmentSize)
    {
	super(1, segmentSize);
    }

    ////////////////////////////////////////////////////////////////////////
    // PUBLIC METHODS
    ////////////////////////////////////////////////////////////////////////

    public Object getValue(int index)
    {
	if (!checkAllocated(index))
	{
	    throw new UnallocatedElementException();
	}

	return getValueUnchecked(index);
    }

    public void setValue(int index, Object value)
    {
	if (!checkAllocated(index))
	{
	    throw new UnallocatedElementException();
	}

	setValueUnchecked(index, value);
    }

    public void clear()
    {
	super.clear();
	m_numAllocatedSegments = 0;
	m_segments = new Object[STARTING_NUM_SEGMENTS][];
    }

    public void free(int index)
    {
	boolean wasAllocated = checkAllocated(index);
	super.free(index);
	if (wasAllocated)
	{
	    setValueUnchecked(index, null);  // Prevent retention of objects.
	}
    }

    ////////////////////////////////////////////////////////////////////////
    // PROTECTED METHODS
    ////////////////////////////////////////////////////////////////////////

    protected int getNumAllocatedSegments()
    {
	return m_numAllocatedSegments;
    }

    protected void allocateSegment()
    {
	if (m_numAllocatedSegments == m_segments.length)
	{
	    expandSegmentsArray();
	}

	m_segments[m_numAllocatedSegments++] = new Object[m_segmentSize];
    }

    ////////////////////////////////////////////////////////////////////////
    // PRIVATE METHODS
    ////////////////////////////////////////////////////////////////////////

    private Object getValueUnchecked(int index)
    {
	int segment = index >> m_segmentSizeBits;
	int offset = index & m_segmentMask;

	return m_segments[segment][offset];
    }

    private void setValueUnchecked(int index, Object value)
    {
	int segment = index >> m_segmentSizeBits;
	int offset = index & m_segmentMask;

	m_segments[segment][offset] = value;
    }

    private void expandSegmentsArray()
    {
	Object[][] oldSegments = m_segments;
	m_segments = new Object[oldSegments.length * 2][];
	System.arraycopy(oldSegments, 0, m_segments, 0, oldSegments.length);
    }

    ////////////////////////////////////////////////////////////////////////
    // PRIVATE CONSTANTS
    ////////////////////////////////////////////////////////////////////////

    private static final boolean DEBUG = true;

    private static final int STARTING_NUM_SEGMENTS = 512;

    ////////////////////////////////////////////////////////////////////////
    // PRIVATE FIELDS
    ////////////////////////////////////////////////////////////////////////

    private int  m_numAllocatedSegments;
    private Object[][]  m_segments = new Object[STARTING_NUM_SEGMENTS][];
}
